@@ -25,7 +25,6 @@ subroutine ddx_get_banner(message, maxlen) bind(C)
     integer(c_int), intent(in), value :: maxlen
     character(len=1, kind=C_char), intent(out) :: message(maxlen)
     integer :: length, i
-    character :: ch
     character (len=4095) :: header
     call get_banner(header)
     message(maxlen) = c_null_char
@@ -90,13 +89,11 @@ end
 !
 function ddx_allocate_model(model, enable_force, solvent_epsilon, solvent_kappa, eta, se, lmax, &
         & n_lebedev, incore, maxiter, jacobi_n_diis, enable_fmm, fmm_multipole_lmax, fmm_local_lmax, &
-        & n_proc, n_spheres, sphere_charges, sphere_centres, &
-        & sphere_radii, length_logfile, c_logfile) result(c_ddx) bind(C)
+        & n_proc, n_spheres, sphere_centres, sphere_radii, length_logfile, c_logfile) result(c_ddx) bind(C)
     integer(c_int), intent(in), value :: model, enable_force, lmax, n_lebedev, maxiter, &
         & incore, jacobi_n_diis, enable_fmm, fmm_multipole_lmax, fmm_local_lmax, n_proc, &
         & n_spheres, length_logfile
-    real(c_double), intent(in) :: sphere_charges(n_spheres), sphere_centres(3, n_spheres), &
-        & sphere_radii(n_spheres)
+    real(c_double), intent(in) :: sphere_centres(3, n_spheres), sphere_radii(n_spheres)
     real(c_double), intent(in), value :: eta, se, solvent_epsilon, solvent_kappa
     !type(c_funptr), value :: printfctn
     type(c_ptr) :: c_ddx
@@ -120,7 +117,7 @@ function ddx_allocate_model(model, enable_force, solvent_epsilon, solvent_kappa,
 
     call params_init(model, enable_force, solvent_epsilon, solvent_kappa, eta, se, lmax, &
         & n_lebedev, incore, maxiter, jacobi_n_diis, enable_fmm, &
-        & fmm_multipole_lmax, fmm_local_lmax, passproc, n_spheres, sphere_charges, &
+        & fmm_multipole_lmax, fmm_local_lmax, passproc, n_spheres, &
         & sphere_centres, sphere_radii, logfile, ddx%params)
     if (ddx%params%error_flag .ne. 0) then
         return
@@ -167,7 +164,6 @@ subroutine ddx_get_error_message(c_ddx, message, maxlen) bind(C)
     character(len=255) :: error_message
     type(ddx_setup), pointer :: ddx
     integer :: length, i
-    character :: ch
     call c_f_pointer(c_ddx, ddx)
     ! Get the actual error message from the collection of structs
     if (ddx%params%error_flag .ne. 0) then
@@ -202,7 +198,6 @@ subroutine ddx_get_logfile(c_ddx, message, maxlen) bind(C)
     character(len=1, kind=C_char), intent(out) :: message(maxlen)
     type(ddx_setup), pointer :: ddx
     integer :: length, i
-    character :: ch
     call c_f_pointer(c_ddx, ddx)
     ! Convert to C message
     message(maxlen) = c_null_char
@@ -349,15 +344,6 @@ function ddx_get_shift(c_ddx) result(c_se) bind(C)
     c_se = ddx % params % se
 end function
 
-subroutine ddx_get_sphere_charges(c_ddx, nsph, c_charge) bind(C)
-    type(c_ptr), intent(in), value :: c_ddx
-    integer(c_int), intent(in), value :: nsph
-    real(c_double), intent(out) :: c_charge(nsph)
-    type(ddx_setup), pointer :: ddx
-    call c_f_pointer(c_ddx, ddx)
-    c_charge(:) = ddx % params % charge(:)
-end subroutine
-
 subroutine ddx_get_sphere_centres(c_ddx, nsph, c_csph) bind(C)
     type(c_ptr), intent(in), value :: c_ddx
     integer(c_int), intent(in), value :: nsph
@@ -439,7 +425,6 @@ subroutine ddx_get_state_error_message(c_state, message, maxlen) bind(C)
     character(len=1, kind=C_char), intent(out) :: message(maxlen)
     character(len=255) :: error_message
     integer :: length, i
-    character :: ch
     call c_f_pointer(c_state, state)
 
     if (state%error_flag .ne. 0) then
@@ -696,8 +681,7 @@ subroutine ddx_multipole_psi(c_ddx, nbasis, nsph, nmultipoles, multipoles, psi)
     integer :: mmax
     call c_f_pointer(c_ddx, ddx)
     mmax = int(sqrt(dble(nmultipoles)) - 1d0)
-    call build_psi(ddx%params, ddx%constants, ddx%workspace, multipoles, &
-        & mmax, psi)
+    call build_psi(ddx%params, multipoles, mmax, psi)
 end
 
 subroutine ddx_multipole_forces(c_ddx, c_state, nsph, ncav, nmultipoles, multipoles, &
